sbit Chip_Select at GPIOD_ODR.B13;
sbit Chip_Select1 at  GPIOA_ODR.B4;
sbit output_sw at GPIOA_ODR.B0;

unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;


// Variable declaration
unsigned int value;
char str[16];
unsigned int input,regout,output;
int vin;

// Get ADC values, returns 0..4095
unsigned int getADC(unsigned short channel)
{

  unsigned int tmp;

  Chip_Select1 = 0;                               // Select MCP3204
  SPI3_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI3_Read(channel) & 0x0F;               // Get first 8 bits of ADC value
  tmp = tmp << 8;                                // Shift ADC value by 8
  tmp = tmp | SPI3_Read(0);                      // Get remaining 4 bits of ADC value
                                                 //   and form 12-bit ADC value
  Chip_Select1= 1;                                // Deselect MCP3204
  return tmp;                                    // Returns 12-bit ADC value

}

// Write measured values to TFT
void processValue(unsigned int pv, unsigned short channel)
{
    char temptext[20];
    char xvalue = 0;
    float tempv = (float) pv;
    tempv/=100.0;
    sprintf(temptext,"%.2f", tempv);

    if (channel == 0){ xvalue = 30;}
    if (channel == 1){ xvalue = 100;}
    if (channel == 2){ xvalue = 200;}

    TFT_Write_Text(temptext, xvalue, 60);

}



// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC)
{
    char temp;
    Chip_Select = 0;                   // Select DAC chip
    // Send High Byte
    temp = (valueDAC >> 8) & 0x0F;     // Store valueDAC[11..8] to temp[3..0]
    temp |= 0x30;                      // Define DAC setting, see MCP4921 datasheet
    SPI3_Write(temp);                  // Send high byte via SPI
    // Send Low Byte
    temp = valueDAC;                   // Store valueDAC[7..0] to temp[7..0]
    SPI3_Write(temp);                  // Send low byte via SPI
    Chip_Select = 1;                   // Deselect DAC chip
}

static void DisplayInit()
{

    TFT_Init_ILI9341_8bit(320, 240);
    TFT_BLED = 1;
    TFT_Set_Pen(CL_WHITE, 1);
    TFT_Set_Brush(1,CL_WHITE,0,0,0,0);

    TFT_Fill_Screen(CL_WHITE);
    TFT_Write_Text("IN", 30, 30);
    TFT_Write_Text("REG", 100, 30);
    TFT_Write_Text("OUT", 200, 30);
}

void main(){

  int set_value, vin;
  bit reset;
  value = 0;

  // Configure DAC chip select line

  GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_13 );
  GPIO_Digital_Output( &GPIOA_BASE, _GPIO_PINMASK_0 | _GPIO_PINMASK_4 );
  GPIO_Digital_Input( &GPIOC_BASE, _GPIO_PINMASK_8 );
  
  GPIO_Digital_Input ( &GPIOA_BASE, _GPIO_PINMASK_9 | _GPIO_PINMASK_10);


  Chip_Select = 1;                   // Deselect DAC
  Chip_Select1 = 1;                 // Deselect MCP3204

  reset = 0;
  vin = 0;
  input = 0;
  regout = 0;
  output = 0;

  set_value = 0;
  // Initialize SPI1 module
  SPI3_Init_Advanced( _SPI_FPCLK_DIV16, _SPI_MASTER | _SPI_8_BIT |
                    _SPI_CLK_IDLE_LOW | _SPI_SECOND_CLK_EDGE_TRANSITION |
                    _SPI_MSB_FIRST | _SPI_SS_DISABLE | _SPI_SSM_ENABLE |
                    _SPI_SSI_1, &_GPIO_MODULE_SPI3_PC10_11_12 );

  // Set inital DAC output value to the mid range

  value = 600;
  set_value = 950;
  output_sw = 1;

  DisplayInit();

  while(1)
  {                                // Endless loop

      TFT_Rectangle(30,60, 300,80);

      if(GPIOC_IDR.B8 == 1){ output_sw = 1;}  // start measuring
      else if(GPIOC_IDR.B8 == 0){ output_sw = 0; reset = 0;}       // stop measuring and reset

      // If A9 button is pressed
      if ((GPIOA_IDR.B9) && (value < 4095))
      {
          set_value++;                              //   increment value
      }

//        If A10 button is pressed
      else if ((GPIOA_IDR.B10) && (value > 0))
      {
              set_value--;                            //   decrement value
      }


      regout = getADC(0)/1.82;       // Get ADC result from Channel 0
      ProcessValue(regout,1);   // Writes measured value to TFT
      Delay_ms(10);                  // Wait 10ms
      input = getADC(2)/1.82;       // Get ADC result from Channel 2
      vin = input - 150;
      ProcessValue(input,0);   // Writes measured value to TFT
      Delay_ms(10);                  // Wait 10ms


      if(reset == 0)
          value = 0;

    if(output_sw == 1)
    {
        reset = 1;

        while((output - set_value) > 50 | (set_value - output) > 50)
        {

            output = getADC(1)/1.82;       // Get ADC result from Channel 1
            if(vin < set_value)
            {
                TFT_Fill_Screen(CL_WHITE);
                TFT_Write_Text("NOT ENOUGH INPUT VOLTS",30,60);
                output_sw = 0;
                while(vin < set_value)
                {
                    input = getADC(2)/1.82;       // Get ADC result from Channel 2
                    vin = input - 200;
                }

                TFT_Fill_Screen(CL_WHITE);
                TFT_Write_Text("IN", 30, 30);
                TFT_Write_Text("REG", 100, 30);
                TFT_Write_Text("OUT", 200, 30);
                output_sw = 1;
            }

            if(output < set_value)
            {
                value = value + 1;
                if(value>4095){value = 4095;}
            }

            if(output > set_value)
            {
                value = value - 1;
                if(value<1){value = 2;}
            }

            DAC_Output(value);
      }

    }

    if(output < set_value)
    {
        value = value + 1;
        if(value>4095) {value = 4095;}
    }

    if(output > set_value)
    {
        value = value - 1;
        if(value<1) {value = 2;}
    }

    DAC_Output(value);
    output = getADC(1)/1.82;
    ProcessValue(output,2);   // Writes measured value to Lcd
    Delay_ms(10);                  // Wait 10ms
  }

}